@@ -1,11 +1,12 @@
 import json
 import threading
 
+import cairo
 import gi
 import pockethernet
 import logging
 
-from pockethernet import WiremapResult, LinkResult, PoEResult, BerResult
+from pockethernet import WiremapResult, LinkResult, PoEResult, BerResult, TdrResult
 
 import wiremapper.discover
 import wiremapper.wiremap
@@ -174,6 +175,15 @@ class Quicktest(threading.Thread):
 
         wiremap = client.get_wiremap()
 
+        GLib.idle_add(self.callback, True, "Running TDR")
+        tdr = client.get_tdr(long_distance=True)
+        if not tdr.valid:
+            tdr = client.get_tdr(long_distance=False)
+
+        spf = ""
+        if sum(tdr.split) > 0:
+            spf = "\nSplit pair detected"
+
         if self.cancelled.is_set():
             GLib.idle_add(self.callback, False, "Stopped", [])
             return
@@ -184,31 +194,31 @@ class Quicktest(threading.Thread):
         # Check the situations that guarantee a wiremap adapter is connected first because it's very fast and
         # guarantees the other tests will fail anyway
         if wiremap.connections == [None, 1, 2, 3, 4, 5, 6, 7, 8] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "Straight cable, unshielded", [wiremap])
+            GLib.idle_add(self.callback, False, "Straight cable, unshielded" + spf, [wiremap, tdr])
             return
 
         if wiremap.connections == [0, 1, 2, 3, 4, 5, 6, 7, 8] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "Straight cable, shielded", [wiremap])
+            GLib.idle_add(self.callback, False, "Straight cable, shielded" + spf, [wiremap, tdr])
             return
 
         if wiremap.connections == [None, 1, 2, 3, None, None, 6, None, None] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "2-pair cable, unshielded", [wiremap])
+            GLib.idle_add(self.callback, False, "2-pair cable, unshielded" + spf, [wiremap, tdr])
             return
 
         if wiremap.connections == [0, 1, 2, 3, None, None, 6, None, None] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "2-pair cable, shielded", [wiremap])
+            GLib.idle_add(self.callback, False, "2-pair cable, shielded" + spf, [wiremap, tdr])
             return
 
         if wiremap.connections == [None, 8, 7, 6, 5, 4, 3, 2, 1] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "Rollover cable (for serial console)", [wiremap])
+            GLib.idle_add(self.callback, False, "Rollover cable (for serial console)" + spf, [wiremap, tdr])
             return
 
         if wiremap.connections == [None, 3, 6, 1, 4, 5, 2, 7, 8] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "Crossover cable, unshielded", [wiremap])
+            GLib.idle_add(self.callback, False, "Crossover cable, unshielded" + spf, [wiremap, tdr])
             return
 
         if wiremap.connections == [0, 3, 6, 1, 4, 5, 2, 7, 8] and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "Crossover cable, shielded", [wiremap])
+            GLib.idle_add(self.callback, False, "Crossover cable, shielded" + spf, [wiremap, tdr])
             return
 
         if wiremap.shorts == [None, 3, 6, None, 7, 8, None, None, None] and self.all_none(wiremap.connections):
@@ -224,7 +234,7 @@ class Quicktest(threading.Thread):
             return
 
         if self.all_none(wiremap.connections) and self.all_none(wiremap.shorts):
-            GLib.idle_add(self.callback, False, "Open", [wiremap])
+            GLib.idle_add(self.callback, False, "Open" + spf, [wiremap, tdr])
             return
 
         # Test a link because weird wiremap reading might just be an ethernet port
@@ -325,18 +335,35 @@ class AppWindow:
         self.application = application
         builder = Gtk.Builder()
         builder.add_from_resource('/nl/brixit/wiremapper/ui/wiremapper.ui')
-        builder.connect_signals(Handler(builder))
+        builder.connect_signals(Handler(builder, self))
+
+        css = Gio.resources_lookup_data("/nl/brixit/wiremapper/ui/style.css", 0)
+
+        self.provider = Gtk.CssProvider()
+        self.provider.load_from_data(css.get_data())
 
         window = builder.get_object("main_window")
         window.set_application(self.application)
+
+        self.apply_css(window, self.provider)
+
         window.show_all()
 
         Gtk.main()
 
+    def apply_css(self, widget, provider):
+        Gtk.StyleContext.add_provider(widget.get_style_context(),
+                                      provider,
+                                      Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+
+        if isinstance(widget, Gtk.Container):
+            widget.forall(self.apply_css, provider)
+
 
 class Handler:
-    def __init__(self, builder):
+    def __init__(self, builder, application):
         self.builder = builder
+        self.application = application
         self.window = builder.get_object('main_window')
         self.mobile_stackswitcher = builder.get_object('mobile_stackswitcher')
         self.quicktest_spinner = builder.get_object('quicktest_spinner')
@@ -471,12 +498,18 @@ class Handler:
                 elif isinstance(block, WiremapResult):
                     box = self.make_result_wiremap(block)
                     self.quicktest_result.pack_start(box, False, False, True)
+                    box.show()
                 elif isinstance(block, PoEResult):
                     box = self.make_result_poe(block)
                     self.quicktest_result.pack_start(box, False, False, True)
                 elif isinstance(block, BerResult):
                     box = self.make_result_ber(block)
                     self.quicktest_result.pack_start(box, False, False, True)
+                elif isinstance(block, TdrResult):
+                    box = self.make_result_tdr(block)
+                    self.quicktest_result.pack_start(box, False, False, True)
+
+            self.application.apply_css(self.quicktest_result, self.application.provider)
             self.quicktest_result.show_all()
 
     def on_custom_start_clicked(self, button):
@@ -557,6 +590,20 @@ class Handler:
         ctx = widget.get_style_context()
         ctx.add_class(new_class)
 
+    def expose_wiremap(self, widget, ctx):
+        print("EXPOSE!")
+        style = widget.get_style_context()
+        state = Gtk.StateFlags(1)
+        font = style.get_font(state)
+        font_color = style.get_color(state)
+
+        ctx.set_font_size(font.get_size() / 1024)
+        ctx.select_font_face(font.get_family(), cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)
+
+        size = widget.get_allocation()
+        wiremapper.wiremap.draw_wiremap_cairo(ctx, width=size.width, height=size.height, connections=widget.connections,
+                                              shorts=widget.shorts, color=font_color)
+
     def make_result_wiremap(self, result):
         if not isinstance(result, WiremapResult):
             return
@@ -576,11 +623,14 @@ class Handler:
         font_color = 'rgb({}, {}, {})'.format(int(font_color.red * 255), int(font_color.green * 255),
                                               int(font_color.blue * 255))
 
-        svg = wiremapper.wiremap.draw_wiremap(result.connections, result.shorts, family=font_face, size=font_size,
-                                              color=font_color)
-        stream = Gio.MemoryInputStream.new_from_bytes(GLib.Bytes.new(svg.encode('UTF-8')))
-        pixbuf = GdkPixbuf.Pixbuf.new_from_stream(stream, None)
-        image = Gtk.Image.new_from_pixbuf(pixbuf)
+        image = Gtk.DrawingArea()
+        image.set_size_request(200, 150)
+        image.connections = result.connections
+        image.shorts = result.shorts
+        image.connect("draw", self.expose_wiremap)
+
+        # svg = wiremapper.wiremap.draw_wiremap(result.connections, result.shorts, family=font_face, size=font_size,
+        #                                      color=font_color)
 
         frame_vbox.pack_start(image, False, False, True)
         frame = Gtk.Frame()
@@ -699,9 +749,13 @@ class Handler:
         grid.attach(Gtk.Label(label="Packets received", xalign=0), 0, 1, 1, 1)
         grid.attach(Gtk.Label(label="Errors", xalign=0), 0, 2, 1, 1)
 
-        sent = "{:n}".format(result.sent)
-        received = "{:n}".format(result.received)
-        errors = "{} ({:.2f}%)".format(result.errors, (result.errors / result.sent) * 100.0)
+        sent = "0"
+        received = "0"
+        errors = "unknown"
+        if result.status:
+            sent = "{:n}".format(result.sent)
+            received = "{:n}".format(result.received)
+            errors = "{} ({:.2f}%)".format(result.errors, (result.errors / result.sent) * 100.0)
 
         grid.attach(Gtk.Label(label=sent, xalign=0), 1, 0, 1, 1)
         grid.attach(Gtk.Label(label=received, xalign=0), 1, 1, 1, 1)
@@ -716,6 +770,65 @@ class Handler:
         box.pack_start(frame, False, False, 12)
         return box
 
+    def add_passive_row(self, listbox, widget):
+        row = Gtk.ListBoxRow(selectable=False, activatable=False)
+        row.add(widget)
+        listbox.insert(row, -1)
+
+    def make_result_tdr(self, result):
+        if not isinstance(result, TdrResult):
+            return
+
+        box = self.make_result('Time Domain Reflectometry')
+
+        frame = Gtk.Frame()
+        listbox = Gtk.ListBox()
+        listbox.set_selection_mode(Gtk.SelectionMode.NONE)
+
+        splits_found = [i for i, x in enumerate(result.split) if x]
+        if len(splits_found) > 0:
+            error_row = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+            error_row.set_margin_start(12)
+            error_row.set_margin_end(12)
+            error_row.set_margin_top(16)
+            error_row.set_margin_bottom(14)
+            if len(splits_found) == 1:
+                text = "Split pair detected in pair {}".format(splits_found[0] + 1)
+            elif len(splits_found) == 2:
+                text = "Split pair detected between pair {} and {}".format(splits_found[0] + 1, splits_found[1] + 1)
+            else:
+                text = "Split pair detected"
+            split_label = Gtk.Label(xalign=0)
+            split_label.set_markup('<b>{}</b>'.format(text))
+            split_label.get_style_context().add_class('error')
+
+            split_label.set_line_wrap(True)
+            error_row.pack_start(split_label, False, False, True)
+            help_label = Gtk.Label(label="Significant crosstalk detected inside the cable, this is either a cable with "
+                                         "split pairs or the cable has no twisting", xalign=0)
+            help_label.set_line_wrap(True)
+            help_label.get_style_context().add_class('error')
+            error_row.pack_start(help_label, False, False, True)
+
+            self.add_passive_row(listbox, error_row)
+            self.add_passive_row(listbox, Gtk.Separator())
+
+        info_row = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        info_row.set_margin_start(12)
+        info_row.set_margin_end(12)
+        info_row.set_margin_top(16)
+        info_row.set_margin_bottom(14)
+        self.add_passive_row(listbox, info_row)
+
+        info_label = Gtk.Label(label="Pair lengths are {}m, {}m, {}m, {}m ".format(*result.distance), xalign=0)
+        info_label.set_line_wrap(True)
+        info_row.pack_start(info_label, False, False, True)
+
+        self.add_class(frame, 'view')
+        frame.add(listbox)
+        box.pack_start(frame, False, False, 12)
+        return box
+
     def on_custom_slot_list_row_activated(self, listbox, row):
         if isinstance(row.get_child(), Gtk.Image):
             self.add_slot()
